/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.servlet.examples.swing;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import org.apithefire.servlet.examples.resource.DirectoryListingExample;
import org.apithefire.servlet.examples.resource.StaticSiteExample;
import org.apithefire.servlet.examples.wicket.PureWicketServletExample;
import org.apithefire.servlet.examples.wicket.ResourceAndWicketFilterExample;
import org.apithefire.servlet.examples.wicket.ResourceAndWicketServletExample;
import org.apithefire.util.lang.Objects;

public class ExampleSelector extends JFrame {
	
	private static class ClassWrapper {
	
		private Class<?> clazz;
		
		public ClassWrapper(Class<?> clazz) {
			this.clazz = Objects.nonNull(clazz);
		}
		
		public Class<?> getWrappedClass() {
			return clazz;
		}
		
		@Override
		public String toString() {
			return clazz.getName();
		}
	
	}
	
	private ClassWrapper[] listData;
	private Console console;

	public ExampleSelector(Collection<Class<?>> exampleClasses, Console console) {
		this.listData = wrapClasses(exampleClasses);
		this.console = console;
		initFrame();
	}
	
	private static ClassWrapper[] wrapClasses(Collection<Class<?>> classes) {
		ClassWrapper[] classWrappers = new ClassWrapper[classes.size()];
		
		int i = 0;
		
		for (Class<?> clazz : classes) {
			classWrappers[i++] = new ClassWrapper(clazz);
		}
		return classWrappers;
	}
	
	private void initFrame() {
		setTitle("Select An Example");
		setResizable(false);
		this.setLocationByPlatform(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		getContentPane().add(createExampleList());
		pack();
	}
	
	private JList createExampleList() {
		final JList list = new JList();
		list.setListData(listData);
		list.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				ClassWrapper value = (ClassWrapper) list.getSelectedValue();
				runExample(value.getWrappedClass());
			}
		});
		return list;
	}
	
	private void runExample(final Class<?> exampleClass) {
		dispose();
		console.setTitle(exampleClass.getSimpleName());
		console.setVisible(true);
		new Thread() {
			@Override
			public void run() {
				try {
					Method mainMethod = exampleClass.getMethod("main",
							new String[0].getClass());
					mainMethod.invoke(null, new Object[] { new String[0] });
				} catch (Exception e) {
					if (e instanceof InvocationTargetException) {
						showErrorMessage(e.getCause().getMessage());
					} else {
						showErrorMessage(e.getMessage());
					}
					console.dispose();
				}
			}
		}.start();
	}
	
	public static void main(String[] args) {
		try {
			Collection<Class<?>> exampleClasses = new ArrayList<Class<?>>();
			
			exampleClasses.add(StaticSiteExample.class);
			exampleClasses.add(DirectoryListingExample.class);
			exampleClasses.add(PureWicketServletExample.class);
			exampleClasses.add(ResourceAndWicketServletExample.class);
			exampleClasses.add(ResourceAndWicketFilterExample.class);
			
			trySetNativeLookAndFeel();
			
			ExampleSelector selector = new ExampleSelector(exampleClasses,
					Console.getInstance());
			
			selector.setVisible(true);
		} catch (Throwable e) {
			showErrorMessage(e.getMessage());
		}
	}
	
	private static void showErrorMessage(String message) {
		JOptionPane.showMessageDialog(null, message, "Error",
				JOptionPane.ERROR_MESSAGE);
	}
	
	private static void trySetNativeLookAndFeel() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			// use back default look and feel
		}
	}
	
}
