/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.servlet.examples.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.URI;
import java.net.UnknownHostException;

import org.apithefire.servlet.examples.swing.Console;
import org.apithefire.util.io.RuntimeIoException;
import org.apithefire.util.lang.Path;
import org.apithefire.util.net.BrowserLauncher;
import org.apithefire.util.nio.Charsets;
import org.apithefire.util.resource.Resource;
import org.apithefire.util.resource.Resources;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExamplesUtil {
	
	private final static Logger logger = LoggerFactory.getLogger(ExamplesUtil.class);
	
	private ExamplesUtil() {
	}

	public static int findUnusedPort() {
		ServerSocket socket = null;
		
		try {
			socket = new ServerSocket(0);
			return socket.getLocalPort();
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		} finally {
			if (socket != null) {
				try {
					socket.close();
				} catch (IOException e) {
					throw new RuntimeIoException(e);
				}
			}
		}
	}
	
	public static void launchBrowser(int port) {
		try {
			InetAddress localhost = InetAddress.getLocalHost();
			String localHostName = localhost.getHostName();
			URI uri = URI.create("http://" + localHostName + ":" + port + "/");
			
			Console console = Console.getInstance();
			
			console.setTitle(console.getTitle() + " @ " + uri
					+ " (close to shutdown server)");
			
			logger.info("Launching browser");
			
			boolean launched = BrowserLauncher.open(uri);
			
			
			if (!launched) {
				logger.info("Failed to launch browser, please manually open "
						+ uri);
			}
		} catch (UnknownHostException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static String loadResource(Class<?> relativeClass, String fileName) {
		try {
			return loadResourceThrows(relativeClass, fileName);
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		}
	}
	
	private static String loadResourceThrows(Class<?> relativeClass,
			String fileName) throws IOException {
		Resource resource = Resources.relativeToClass(relativeClass)
				.getResource(new Path(fileName));
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				resource.openFile(), Charsets.UTF_8));

		try {
			StringBuilder description = new StringBuilder();
			String line = reader.readLine();

			while (line != null) {
				description.append(line).append('\n');
				line = reader.readLine();
			}
			return description.toString();
		} finally {
			reader.close();
		}
	}
	
}
