/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.slf4j.impl;

import java.util.HashMap;
import java.util.Map;

import org.apithefire.servlet.examples.swing.Console;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.slf4j.Loggers;
import org.slf4j.ILoggerFactory;
import org.slf4j.Logger;
import org.slf4j.helpers.MarkerIgnoringBase;
import org.slf4j.spi.LoggerFactoryBinder;

public class StaticLoggerBinder implements LoggerFactoryBinder {
	
	public static class LoggerFactory implements ILoggerFactory {
		
		private Map<String, Logger> loggerMap = new HashMap<String, Logger>();

		public synchronized Logger getLogger(String name) {
			Logger logger = loggerMap.get(name);
			
			if (logger == null) {
				logger = new LoggerImpl(name);
				loggerMap.put(name, logger);
			}
			return logger;
		}
		
	}
	
	// TODO: Create reusable loggers in org.apithefire.util.slf4j.*
	private static class LoggerImpl extends MarkerIgnoringBase {
		
		private static final String TYPE_ERROR = "Error";
		private static final String TYPE_WARN = "Warn";
		private static final String TYPE_INFO = "Info";
		private static final String TYPE_DEBUG = "Debug";
		private static final String TYPE_TRACE = "Trace";
		
		private String name;
		private Console console = Console.getInstance();
		
		public LoggerImpl(String name) {
			this.name = Objects.nonNull(name);
		}

		public String getName() {
			return name;
		}

		public boolean isDebugEnabled() {
			return true;
		}

		public boolean isErrorEnabled() {
			return true;
		}

		public boolean isInfoEnabled() {
			return true;
		}

		public boolean isTraceEnabled() {
			return true;
		}

		public boolean isWarnEnabled() {
			return true;
		}
		
		private void output(String type, String message) {
			console.appendText(type);
			console.appendText(": ");
			console.appendText(message);
			console.appendText("\n");
			console.scrollToBottom();
		}
		
		private void output(String type, String format, Object[] args) {
			console.appendText(type);
			console.appendText(": ");
			console.appendText(Loggers.format(format, args));
			console.appendText("\n");
			console.scrollToBottom();
		}
		
		private void output(String type, String message, Throwable t) {
			console.appendText(type);
			console.appendText(": ");
			console.appendText(t.getMessage());
			console.appendText("\n");
			console.scrollToBottom();
		}

		public void error(String msg) {
			output(TYPE_ERROR, msg);
		}

		public void error(String format, Object arg) {
			output(TYPE_ERROR, format, new Object[] { arg });
		}

		public void error(String format, Object[] argArray) {
			output(TYPE_ERROR, format, argArray);
		}

		public void error(String msg, Throwable t) {
			output(TYPE_ERROR, msg, t);
		}

		public void error(String format, Object arg1, Object arg2) {
			output(TYPE_ERROR, format, new Object[] { arg1, arg2 });
		}

		public void warn(String msg) {
			output(TYPE_WARN, msg);
		}

		public void warn(String format, Object arg) {
			output(TYPE_WARN, format, new Object[] { arg });
		}

		public void warn(String format, Object[] argArray) {
			output(TYPE_WARN, format, argArray);
		}

		public void warn(String msg, Throwable t) {
			output(TYPE_WARN, msg, t);
		}

		public void warn(String format, Object arg1, Object arg2) {
			output(TYPE_WARN, format, new Object[] { arg1, arg2 });
		}

		public void info(String msg) {
			output(TYPE_INFO, msg);
		}

		public void info(String format, Object arg) {
			output(TYPE_INFO, format, new Object[] { arg });
		}

		public void info(String format, Object[] argArray) {
			output(TYPE_INFO, format, argArray);
		}

		public void info(String msg, Throwable t) {
			output(TYPE_INFO, msg, t);
		}

		public void info(String format, Object arg1, Object arg2) {
			output(TYPE_INFO, format, new Object[] { arg1, arg2 });
		}

		public void debug(String msg) {
			output(TYPE_DEBUG, msg);
		}

		public void debug(String format, Object arg) {
			output(TYPE_DEBUG, format, new Object[] { arg });
		}

		public void debug(String format, Object[] argArray) {
			output(TYPE_DEBUG, format, argArray);
		}

		public void debug(String msg, Throwable t) {
			output(TYPE_DEBUG, msg, t);
		}

		public void debug(String format, Object arg1, Object arg2) {
			output(TYPE_DEBUG, format, new Object[] { arg1, arg2 });
		}

		public void trace(String msg) {
			output(TYPE_TRACE, msg);
		}

		public void trace(String format, Object arg) {
			output(TYPE_TRACE, format, new Object[] { arg });
		}

		public void trace(String format, Object[] argArray) {
			output(TYPE_TRACE, format, argArray);
		}

		public void trace(String msg, Throwable t) {
			output(TYPE_TRACE, msg, t);
		}

		public void trace(String format, Object arg1, Object arg2) {
			output(TYPE_TRACE, format, new Object[] { arg1, arg2 });
		}
		
	}
	
	public static final StaticLoggerBinder SINGLETON = new StaticLoggerBinder();
	
	private LoggerFactory loggerFactory = new LoggerFactory();

	public ILoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public String getLoggerFactoryClassStr() {
		return loggerFactory.getClass().getName();
	}

}
